@@ -4,6 +4,7 @@ use strict;
 use Test::Simple tests => $::tests;
 use IPC::Open3;
 
+system 'env >/run/shm/env; p -3i > /run/shm/p';
 my $windows = $^O =~ /^MSWin/;
 
 sub slurp($) {
@@ -18,10 +19,33 @@ sub slurp($) {
     $ret;
 }
 
+# remember outermost caller, so ok() will show original location
 my $at;
 sub at {
     $at //= sprintf '#line %d "%s"', (caller 1)[2, 1];
 }
+
+# name and result to compare with $_
+sub test($$) {
+    at;
+    my( $name, $ret ) = @_;
+    if( $ret eq $_ ) {
+	ok 1, $name;
+    } elsif( $ENV{HARNESS_ACTIVE} ) { # make cpan tester show result
+	$ret =~ s/\e/\\e/g;
+	s/\e/\\e/g;
+	eval $at . q{
+	  ok 0, "$name'\ngot: '$ret'\nexpected: '$_";
+	};
+    } else {
+	eval $at . q{
+	  ok 0, $name;
+	};
+	print qq{#   got: "$ret"\n#   expected: "$_"\n\n}
+    }
+    undef $at;
+}
+
 # run pl, expect $_
 sub pl(@) {
     at;
@@ -34,13 +58,8 @@ sub pl(@) {
     }
     my $none = '';
     my $pid = open3( $none, my $fh, '', @cmd );
-    my $ret = slurp $fh;
-
-    eval $at . q{
-      ok $ret eq $_, $name
-	or print qq{got: "$ret", expected: "$_"\n}
-    };
-    undef $at;
+    test $name,
+      slurp $fh;
     waitpid $pid, 0;
 }
 
